(*****************************************************************
 * Copyright (C), 2000 Pawel Ziemian, All rights reserved        *
 *****************************************************************)

(*****************************************************************
 *
 * SOURCE FILE: xlat2xlt.pas
 *
 * MODULE NAME: Windows XLAT file to Dos Navigator XLT file
 *              Converter
 *
 * PURPOSE:     This program converts windows xlat###.bin file
 *              to Dos Navigator's win###.xlt file.
 *
 * AUTHOR:      Pawel Ziemian (PZ)
 *
 * REVIEWED BY:
 *
 * HISTORY:     Ver   Date       Sign   Description
 *
 *              001   2000.11.15 PZ     Created
 *
 *****************************************************************)
 
{$A+,B-,D+,E-,F-,G+,I+,L+,N+,O-,P-,Q-,R-,S+,T-,V+,X+,Y+}
{$M 16384,0,0}

program XLAT2XLT;

function Dump ( Size : Word; Page : Word; Data : array of Byte ) : Integer;
var
  n : string;
  f : file of Byte;
  i : Byte;
begin
  Dump := -10;
  Str ( Page, n );
  n := 'win'+n+'.xlt';
  Writeln ( 'Creating ', n, ' file.' );
  Assign ( f, n );
  Rewrite ( f );
  if size = $140 then begin
    for i := $00 to $1F do begin
      if Data[i] <> i then begin
        Write ( f, i, Data[i] );
      end;
    end;
    for i := $80 to $FF do begin
      if Data[i-$80+$20] <> i then begin
        Write ( f, i, Data[i-$80+$20] );
      end;
    end;
    Dump := 0;
  end;
  Close ( f );
end;

function Main ( const Filename : string ) : Integer;
var
  input : file;
  fsize : Longint;
  size  : Word;
  page  : Word;
  s     : Word;
  data  : array [0..511] of Byte;
begin
  Assign ( input, Filename );
  Reset ( input, 1 );
  fsize := FileSize ( input );
  Main := -1;
  if fsize >= 4 then begin
    BlockRead ( input, size, SizeOf(size), s );
    Main := -2;
    if s = SizeOf(size) then begin
      Main := -3;
      if size = fsize-2 then begin
        BlockRead ( input, page, SizeOf(size), s );
        Main := -4;
        if s = SizeOf(size) then begin
          Main := -5;
          Dec ( size, 2 );
          if size <= SizeOf(data) then begin
            BlockRead ( input, data, size, s );
            Main := -6;
            if s = size then begin
              Main := Dump ( size, page, data );
            end;
          end;
        end;
      end;
    end;
  end;
  Close ( input );
end;

procedure Logo;
begin
  Writeln;
  Writeln ( 'Windows XLAT file to Dos Navigator XLT file Converter' );
  Writeln ( 'Version 1.0' );
  Writeln;
end;

procedure Usage;
begin
  Writeln ( 'Usage:' );
  Writeln ( '  XLAT2XLT <xlat###.bin>' );
end;

procedure Error ( Code : Integer );
var
  text : string;
begin
  case Code of
    0:
      text := 'Successful';
    -1,-3,-5,-10:
      text := 'Invalid input file';
    -2,-4,-6:
      text := 'Error reading input file';
  else
    Str ( Code, text );
    text := 'Error code ' + text;
  end;
  Writeln ( text, '.' );
end;

begin
  Logo;
  if ParamCount = 1 then
    Error ( Main ( ParamStr ( 1 ) ) )
  else
    Usage;
  Writeln;
end.
